@@ -19,6 +19,7 @@ import compiler
 import os
 import re
 import sys
+import _ast
 
 import pep8
 import pyflakes.checker
@@ -27,11 +28,11 @@ from hghooks import CheckerManager, re_options
 
 
 def pep8_checker(ignore=''):
-    ignore_list = []
+    extra_args = []
     if ignore:
-        ignore_list = ['--ignore=%s' % ign.strip(',')
-                       for ign in ignore.split()
-                       if ign]
+        ignore_spec = ','.join([ign.strip(', ') for ign in ignore.split()
+                                if ign.strip(', ')])
+        extra_args.append('--ignore=%s' % ignore_spec)
 
     def check_pep8(files_to_check, msg):
         for filename, data in files_to_check.items():
@@ -42,7 +43,7 @@ def pep8_checker(ignore=''):
 
         # monkey patch sys.argv options so we can call pep8
         old_args = sys.argv
-        sys.argv = ['pep8'] + files_to_check.keys() + ignore_list
+        sys.argv = ['pep8'] + files_to_check.keys() + extra_args
         options, args = pep8.process_options()
         sys.argv = old_args
         for path in args:
@@ -79,6 +80,10 @@ def pdbhook(ui, repo, hooktype, node, pending, **kwargs):
     return checker_manager.check(pdb_checker)
 
 
+def pyflakes_version():
+    return tuple([int(part) for part in pyflakes.__version__.split('.')])
+
+
 def pyflakes_check(data, filename):
     """
     Check the Python source given by C{data} for flakes.
@@ -86,21 +91,12 @@ def pyflakes_check(data, filename):
     It's like pyflakes.scripts.check function but ignores those lines which
     ends with ends with a "pyflakes:ignore" comment.
 
-    Code adapted from http://djangosnippets.org/snippets/1762/ which is adapted
-    from pyflakes.scripts.pyflakes.check function.
+    Code adapted from pyflakes.scripts.pyflakes.check function.
     """
-    # Since compiler.parse does not reliably report syntax errors, use the
-    # built in compiler first to detect those.
+    # First, compile into an AST and handle syntax errors.
     try:
-        try:
-            compile(data, filename, "exec")
-        except MemoryError:
-            # Python 2.4 will raise MemoryError if the source can't be
-            # decoded.
-            if sys.version_info[:2] == (2, 4):
-                raise SyntaxError(None)
-            raise
-    except (SyntaxError, IndentationError), value:
+        tree = compile(data, filename, "exec", _ast.PyCF_ONLY_AST)
+    except SyntaxError, value:
         msg = value.args[0]
 
         (lineno, offset, text) = value.lineno, value.offset, value.text
@@ -118,11 +114,17 @@ def pyflakes_check(data, filename):
                 offset = offset - (len(text) - len(line))
 
             print >> sys.stderr, '%s:%d: %s' % (filename, lineno, msg)
+            print >> sys.stderr, line
+
+            if offset is not None:
+                print >> sys.stderr, " " * offset, "^"
+
         return 1
     else:
-        # Okay, it's syntactically valid.  Now parse it into an ast and check
-        # it.
-        tree = compiler.parse(data)
+        if pyflakes_version() < (0, 5, 0):
+            tree = compiler.parse(data)
+
+        # Okay, it's syntactically valid.  Now check it.
         w = pyflakes.checker.Checker(tree, filename)
 
         lines = data.split('\n')
